%The aim of this program is to compare a model with an analytical solution given in MacAyeal EISMINT : Lessons in Ice-Sheet Modeling
printingflag=false;

numlayers=10;
resolution=30000;

%To begin with the numerical model
md=model;
md=roundmesh(md,750000,resolution);
md=geography(md,'',''); %We can not test iceshelves nor ice rises with this analytical solution
md=parameterize(md,'../Par/RoundSheetStaticEISMINT.par');

%Calculation of the analytical 2d velocity field
constant=0.3;
vx_obs=constant/2*md.x.*(md.thickness).^-1;
vy_obs=constant/2*md.y.*(md.thickness).^-1;
vel_obs=(sqrt((md.vx_obs).^2+(md.vy_obs).^2));

%We extrude the model to have a 3d model
md=extrude(md,numlayers,1);
md=setelementstype(md,'pattyn','all');

%Spc the grids on the bed
pos=find(md.gridonbed);
md.spcvelocity(pos,1:3)=1;

%Now we can solve the problem 
md=solve(md,'analysis_type',DiagnosticSolutionEnum);

%Calculate the depth averaged velocity field (2d):
vx=PatchToVec(md.results.DiagnosticSolution.Vx);
vy=PatchToVec(md.results.DiagnosticSolution.Vy);
vel=zeros(md.numberofgrids2d,1);

grid_vel=0;
for i=1:md.numberofgrids2d
	for j=1:(md.numlayers-1)
		grid_vel=grid_vel+1/(2*(md.numlayers-1))*(sqrt(vx(i+j*md.numberofgrids2d,1).^2+...
			vy(i+j*md.numberofgrids2d,1).^2)+...
			sqrt(vx(i+(j-1)*md.numberofgrids2d,1).^2+vy(i+(j-1)*md.numberofgrids2d,1).^2));
	end
	vel(i,1)=grid_vel;
	grid_vel=0;
end

%Plot of the velocity from the exact and calculated solutions
figure(1)
subplot(2,2,1)
p=patch('Faces',md.elements2d,'Vertices',[md.x2d md.y2d],'FaceVertexCData',...
vel,'FaceColor','interp','EdgeColor','none');
title('Modelled velocity','FontSize',14,'FontWeight','bold')
colorbar; 
caxis([0 200]);
   
subplot(2,2,2)
p=patch('Faces',md.elements2d,'Vertices',[md.x2d md.y2d],'FaceVertexCData',...
vel_obs,'FaceColor','interp','EdgeColor','none');
title('Analytical velocity','FontSize',14,'FontWeight','bold')
colorbar; 
caxis([0 200]);

subplot(2,2,3)
hold on;
plot(sqrt((md.x(1:md.numberofgrids2d)).^2+(md.y(1:md.numberofgrids2d)).^2),vel,'r.');
plot(sqrt((md.x2d).^2+(md.y2d).^2),vel_obs,'b.');
title('Analytical vs calculated velocity','FontSize',14,'FontWeight','bold');
xlabel('distance to the center of the icesheet [m]','FontSize',14,'FontWeight','bold');
ylabel('velocity [m/yr]','FontSize',14,'FontWeight','bold');
legend('calculated velocity','exact velocity');
axis([0 750000 0 200]);
hold off;

subplot(2,2,4)
p=patch('Faces',md.elements2d,'Vertices',[md.x2d md.y2d],'FaceVertexCData',...
abs(vel-vel_obs)./vel_obs*100,'FaceColor','interp','EdgeColor','none');
title('Relative misfit [%]','FontSize',14,'FontWeight','bold')
colorbar;
caxis([0 100]);

if printingflag, 
	set(gcf,'Color','w')
	printmodel('pattynstatic','png','margin','on','marginsize',25,'frame','off','resolution',0.7,'hardcopy','off');
	system(['mv pattynstatic.png ' ISSM_DIR '/../website/doc_pdf/validation/Images/EISMINT/IceSheet']);
end

%Fields and tolerances to track changes
field_names     ={ ...
	'Vx','Vy','Vel', ...
};
field_tolerances={...
	1e-13,1e-13,1e-13, ...
};
field_values={
	vx,vy,vel, ...
};
